@@ -1,7 +1,6 @@
 #!/usr/bin/env python
 
-from __future__ import (absolute_import, division, print_function)
-__metaclass__ = type
+from __future__ import annotations
 
 from setuptools import setup, find_packages
 
